/*****************************************************************************
IsoHex24_4.cpp
Ernest S. Pazera
15DEC2000
Start a WIN32 Application Workspace, add in this file
Needs ddraw.lib, d3d8.lib and dxguid.lib
Needs GDICanvas.h/cpp
Needs DDFuncs.h/cpp
Needs D3DFuncs.h/cpp
*****************************************************************************/

//////////////////////////////////////////////////////////////////////////////
//INCLUDES
//////////////////////////////////////////////////////////////////////////////
#define WIN32_LEAN_AND_MEAN  

#include <windows.h>   
#include <math.h>//sin and cos
#include "GDICanvas.h"
#include "ddraw.h"
#include "DDFuncs.h"
#include "d3d.h"
#include "d3dfuncs.h"

//////////////////////////////////////////////////////////////////////////////
//DEFINES
//////////////////////////////////////////////////////////////////////////////
//name for our window class
#define WINDOWCLASS "ISOHEX25"
//title of the application
#define WINDOWTITLE "IsoHex 25-4"

//screen attributes
const DWORD SCREENWIDTH=640;
const DWORD SCREENHEIGHT=480;
const DWORD SCREENBPP=16;

//tile dimensions
const DWORD TILEWIDTH=64;
const DWORD TILEHEIGHT=32;

//map dimensions
const DWORD MAPWIDTH=20;
const DWORD MAPHEIGHT=40;



//////////////////////////////////////////////////////////////////////////////
//PROTOTYPES
//////////////////////////////////////////////////////////////////////////////
bool Prog_Init();//game data initalizer
void Prog_Loop();//main game loop
void Prog_Done();//game clean up

//////////////////////////////////////////////////////////////////////////////
//GLOBALS
//////////////////////////////////////////////////////////////////////////////
HINSTANCE hInstMain=NULL;//main application handle
HWND hWndMain=NULL;//handle to our main window

//IDirectDraw7 Pointer
LPDIRECTDRAW7 lpdd=NULL;

//surfaces
LPDIRECTDRAWSURFACE7 lpddsPrime=NULL;
LPDIRECTDRAWSURFACE7 lpddsBack=NULL;
LPDIRECTDRAWSURFACE7 lpddsTexture=NULL;

//IDirect3D7
LPDIRECT3D7 lpd3d=NULL;

//IDirect3DDevice
LPDIRECT3DDEVICE7 lpd3ddev=NULL;

//vertices
D3DTLVERTEX vert[4];//three vertices

//////////////////////////////////////////////////////////////////////////////
//WINDOWPROC
//////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK TheWindowProc(HWND hwnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	//which message did we get?
	switch(uMsg)
	{
	case WM_KEYDOWN:
		{
			//check for escape key
			if(wParam==VK_ESCAPE)
			{
				DestroyWindow(hWndMain);
			}

			return(0);//handled message
		}break;
	case WM_DESTROY://the window is being destroyed
		{

			//tell the application we are quitting
			PostQuitMessage(0);

			//handled message, so return 0
			return(0);

		}break;
	case WM_PAINT://the window needs repainting
		{
			//a variable needed for painting information
			PAINTSTRUCT ps;
			
			//start painting
			HDC hdc=BeginPaint(hwnd,&ps);

			/////////////////////////////
			//painting code would go here
			/////////////////////////////

			//end painting
			EndPaint(hwnd,&ps);
						
			//handled message, so return 0
			return(0);
		}break;
	}

	//pass along any other message to default message handler
	return(DefWindowProc(hwnd,uMsg,wParam,lParam));
}


//////////////////////////////////////////////////////////////////////////////
//WINMAIN
//////////////////////////////////////////////////////////////////////////////
int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nShowCmd)
{
	//assign instance to global variable
	hInstMain=hInstance;

	//create window class
	WNDCLASSEX wcx;

	//set the size of the structure
	wcx.cbSize=sizeof(WNDCLASSEX);

	//class style
	wcx.style=CS_OWNDC | CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;

	//window procedure
	wcx.lpfnWndProc=TheWindowProc;

	//class extra
	wcx.cbClsExtra=0;

	//window extra
	wcx.cbWndExtra=0;

	//application handle
	wcx.hInstance=hInstMain;

	//icon
	wcx.hIcon=LoadIcon(NULL,IDI_APPLICATION);

	//cursor
	wcx.hCursor=LoadCursor(NULL,IDC_ARROW);

	//background color
	wcx.hbrBackground=(HBRUSH)GetStockObject(BLACK_BRUSH);

	//menu
	wcx.lpszMenuName=NULL;

	//class name
	wcx.lpszClassName=WINDOWCLASS;

	//small icon
	wcx.hIconSm=NULL;

	//register the window class, return 0 if not successful
	if(!RegisterClassEx(&wcx)) return(0);

	//create main window
	hWndMain=CreateWindowEx(0,WINDOWCLASS,WINDOWTITLE, WS_POPUP | WS_VISIBLE,0,0,320,240,NULL,NULL,hInstMain,NULL);

	//error check
	if(!hWndMain) return(0);

	//if program initialization failed, then return with 0
	if(!Prog_Init()) return(0);

	//message structure
	MSG msg;

	//message pump
	for(;;)	
	{
		//look for a message
		if(PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{
			//there is a message

			//check that we arent quitting
			if(msg.message==WM_QUIT) break;
			
			//translate message
			TranslateMessage(&msg);

			//dispatch message
			DispatchMessage(&msg);
		}

		//run main game loop
		Prog_Loop();
	}
	
	//clean up program data
	Prog_Done();

	//return the wparam from the WM_QUIT message
	return(msg.wParam);
}

//////////////////////////////////////////////////////////////////////////////
//INITIALIZATION
//////////////////////////////////////////////////////////////////////////////
bool Prog_Init()
{
	lpdd=LPDD_Create(hWndMain,DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN | DDSCL_ALLOWREBOOT);

	//set the display mode
	lpdd->SetDisplayMode(SCREENWIDTH,SCREENHEIGHT,SCREENBPP,0,0);

	//create primary surface
	lpddsPrime=LPDDS_CreatePrimary3D(lpdd,1);

	//create back buffer
	lpddsBack=LPDDS_GetSecondary3D(lpddsPrime);
	
	//get the idirect3d pointer
	lpd3d=LPD3D_Create(lpdd);

	//create the idirect3ddevice(hack method)
	lpd3ddev=LPD3DDEV_Create(lpd3d,lpddsBack);

	//set up viewport
	LPD3DDEV_SetViewport(lpd3ddev,0,0,SCREENWIDTH,SCREENHEIGHT);

	//load in texture image
	CGDICanvas gdic;
	gdic.Load(NULL,"texture.bmp");

	//load texture
	lpddsTexture=LPDDS_CreateTexture(lpdd,gdic.GetWidth(),gdic.GetHeight());

	//grab texture's DC
	HDC hdc;
	lpddsTexture->GetDC(&hdc);

	//blit image
	BitBlt(hdc,0,0,gdic.GetWidth(),gdic.GetHeight(),gdic,0,0,SRCCOPY);

	//release texture's dc
	lpddsTexture->ReleaseDC(hdc);

	//set this texture
	lpd3ddev->SetTexture(0,lpddsTexture);


	return(true);//return success
}

//////////////////////////////////////////////////////////////////////////////
//CLEANUP
//////////////////////////////////////////////////////////////////////////////
void Prog_Done()
{	
	//release texture
	LPDDS_Release(&lpddsTexture);

	//release IDirect3DDevice
	LPD3DDEV_Release(&lpd3ddev);

	//release IDirect3D 
	LPD3D_Release(&lpd3d);

	//clean up primary surface(this will clean up the back buffer, also)
	LPDDS_Release(&lpddsPrime);

	//clean up the dd pointer
	LPDD_Release(&lpdd);
}

//////////////////////////////////////////////////////////////////////////////
//MAIN GAME LOOP
//////////////////////////////////////////////////////////////////////////////
void Prog_Loop()
{
	//clear the viewport to black
	lpd3ddev->Clear(0,NULL,D3DCLEAR_TARGET,0,0,0);

	//grab the mouse position
	POINT ptMouse;
	GetCursorPos(&ptMouse);

	//convert mouse position to floating point values
	D3DVALUE MouseX=(D3DVALUE)ptMouse.x;
	D3DVALUE MouseY=(D3DVALUE)ptMouse.y;

	//declare vector x and y for lighting calculations
	D3DVALUE VectorX;
	D3DVALUE VectorY;

	//Distance
	float distance;

	//start the scene
	lpd3ddev->BeginScene();

	//center positions
	D3DVALUE CenterX,CenterY;

	//loop through map
	for(int y=0;y<MAPHEIGHT;y++)
	{
		for(int x=0;x<MAPWIDTH;x++)
		{
			//calculate world coordinates for center of tile
			CenterX=(float)(x*TILEWIDTH+(y&1)*(TILEWIDTH/2));
			CenterY=(float)(y*(TILEHEIGHT/2));

			//set up the vertex
			//v1
			VERTEX_Set(&vert[0],CenterX-TILEWIDTH/2,CenterY,D3DRGB(1.0,1.0,1.0),0.0,0.0);
			//distance calculation
			VectorX=vert[0].sx;
			VectorY=vert[0].sy;
			distance=sqrt((VectorX-MouseX)*(VectorX-MouseX)+(VectorY-MouseY)*(VectorY-MouseY))+1.0;
			//max distance of 128
			if(distance>128.0) distance=128.0;
			//convert distance to 0.0 to 0.5
			distance/=256.0;
			//change color
			vert[0].color=D3DRGB(1.0-distance,1.0-distance,1.0-distance);

			//v2
			VERTEX_Set(&vert[1],CenterX,CenterY-TILEHEIGHT/2,D3DRGB(1.0,1.0,1.0),1.0,0.0);
			//distance calculation
			VectorX=vert[1].sx;
			VectorY=vert[1].sy;
			distance=sqrt((VectorX-MouseX)*(VectorX-MouseX)+(VectorY-MouseY)*(VectorY-MouseY))+1.0;
			//max distance of 128
			if(distance>128.0) distance=128.0;
			//convert distance to 0.0 to 0.5
			distance/=256.0;
			//change color
			vert[1].color=D3DRGB(1.0-distance,1.0-distance,1.0-distance);

			//v3
			VERTEX_Set(&vert[2],CenterX,CenterY+TILEHEIGHT/2,D3DRGB(1.0,1.0,1.0),0.0,1.0);
			//distance calculation
			VectorX=vert[2].sx;
			VectorY=vert[2].sy;
			distance=sqrt((VectorX-MouseX)*(VectorX-MouseX)+(VectorY-MouseY)*(VectorY-MouseY))+1.0;
			//max distance of 128
			if(distance>128.0) distance=128.0;
			//convert distance to 0.0 to 0.5
			distance/=256.0;
			//change color
			vert[2].color=D3DRGB(1.0-distance,1.0-distance,1.0-distance);

			//v4
			VERTEX_Set(&vert[3],CenterX+TILEWIDTH/2,CenterY,D3DRGB(1.0,1.0,1.0),1.0,1.0);
			//distance calculation
			VectorX=vert[3].sx;
			VectorY=vert[3].sy;
			distance=sqrt((VectorX-MouseX)*(VectorX-MouseX)+(VectorY-MouseY)*(VectorY-MouseY));
			//max distance of 128
			if(distance>128.0) distance=128.0;
			//convert distance to 0.0 to 0.5
			distance/=256.0;
			//change color
			vert[3].color=D3DRGB(1.0-distance,1.0-distance,1.0-distance);


			//render the triangle strip
			LPD3DDEV_DrawTriangleStrip(lpd3ddev,vert,4);
		}
	}

	//end the scene
	lpd3ddev->EndScene();

	//flip 
	lpddsPrime->Flip(NULL,DDFLIP_WAIT);
}

